;***********************************************************************
;* Initialition ATMega8 and basic macro                                *
;*                                                                     *
;* Vladimir Visnovsky                                                  *
;***********************************************************************

;  Mapa pinov MCU ------------------------------------------------------ 
; 				-Reset/PC6 		= 1		28 =		PC5/ADC5/SCL*
;				  RXD /PD0		= 2		27 =		PC4/ADC5/SDA*
;				  TXD /PD1		= 3		26 =		PC3/ADC3*
;				 *Int0/PD2		= 4		25 =		PC2/ADC2*
;				 *Int1/PD3		= 5		24 =		PC1/ADC1*
;				XCK/T0/PD4		= 6		23 =		PC0/ADC0*
;				  	   Ucc		= 7		22 =		Gnd
;				  	   Gnd		= 8		21 =		ARef
;			  X1/TOSC1/PB6		= 9		20 =		AUcc*
;			  X2/TOSC2/PB7		= 10	19 =		PB5/*SCK
;	*595_Data 		T1/PD5		= 11	18 =		PB4/*MISO
;	*595_Clk  	  AIN0/PD6		= 12	17 =		PB3/*MOSI
;	*595_Strobe	  AIN1/PD7		= 13	16 =		PB2/OC1b/SS
;			  	  ICP1/PB0		= 14	15 =		PB1/OC1a

; Volne registre
; R0  - R15 - Volne
; R16 - R19 - A,B,T,F
; R20 - R23 - Volne
; R24 - R31 - HL, X, Y, Z

; == Makra ============================================================= 
; Utilita pre prerusenie -------------------------------------------- 2.
;  - PushA - Ulozi do Stacku A,Z,SREG
;  - PopA  - Vytiahne zo Stacku A,Z,SREG
;   Interupt:
;    Pusha
;     LDI A, ..
;		:
;    Popa
;    Reti
;
; Timer Casovac 16 bit ---------------------------------------------- 3.
; - Timer_Init ( HL : Word ) - Spustenie pocitadla CTC. 
; - Timer_Reset        - Vynulovanie pocitacich registrov
; - Timer_Div1         - Preddelicka na /1
; - Timer_Div8          :
; - Timer_Div64         :
; - Timer_Div256        :
; - Timer_Div1024      - Preddelicka na /1024
; Timer Casovac 8 bit
; - Timer8_Init ( L : Byte )
; - Timer8_Reset
; - Timer8_Div1
; - Timer8_Div8
; - Timer8_Div32
; - Timer8_Div64
; - Timer8_Div128
; - Timer8_Div256
; - Timer8_Div1024
;   InitializationTimer:   
;    LDI	H,0
;    LDI    L,100 ; 1Mhz/100=10Khz
;    Timer_Init
;    Timer_Div8   ; Toto uz obsahuje Init 
;    Timer_Reset  ; Toto uz obsahuje Init
;    :
;
; Posuvny register 595 ---------------------------------------------- 4.
; - 595_Init               - prepise porty na vystup
; - 595_Send ( A : Byte )  - odosle byte z A
;   ^ Je volanie cez RCall. Samotny kod sa vola 595_Send_Macro  
; - 595_Done               - Po odoslani generuje impulz STB
;   ^ Je volanie cez RCall. Samotny kod sa vola 595_Done_Macro  
;   595_Init
;   SendTo74HCT595:
;    LDI	A, 0x55
;    595_Send
;      : 
;    595_Send
;    595_Done
;
; Nastavenie vonkajsieho prerusenia --------------------------------- 5.
; - Int0_Enable		- Povolenie INT0
; - Int0_Disable	- Zakazanie INT0
; - Int0_OnH		-- Prerusenie od kladneho impulzu
; - Int0_OnL     	-- Prerusenie od zapurnrho impulzu
; - Int0_OnChange   -- Prerusenie oz zmeny potencialu
; - Int1_Enable
; - Int1_Disable
; - Int1_OnH
; - Int1_OnL
; - Int1_OnChange
; EnableInteruptOnPinINT0:
;  Int0_Enable
;  Int0_OnH (Predvolene v procedure Enable )
;
; A/D Prevodnik ----------------------------------------------------- 6.
; - AD_Init - 8bit/5V/0.pin
; - AD_Mux ( A : Byte - Vstupny pin )
; - AD_Read : A - Vystupna 8. bitova hodnota 
; - AD_Read10 : HL - Vystup 10. bitova hodnota
; - AD_8bit
; - AD_10bit
; - AD_Ref5V
; - AD_Ref2V56
; - AD_Reset
; - AD_Start
; - AD_Wait
; - AD_Convert ( A : Byte - Vstupny pin ) : A - Vystupna hodnota
;   ^ Volanie RCall wAD_Read.
; - AD_Convert10 ( A : Byte - Vstupny pin ) : HL - Vystupna hodnota
;   ^ Volanie RCall wAD_Read10
;  InitAD:
;   AD_Init ;8bit/5V/Pin0
;  Convert_10Bit_2,56V_2.pin:
;   AD_10Bit
;   AD_Ref2V56
;   Ldi A , 2 ; 2. pin
;   AD_Convert10
;   ; V HL je vystupna hodnota
;
; EEPROM ------------------------------------------------------------ 7.
; - EEPROM_Wait   - Cakaj kym je pamat volna
; - EEPROM_Read   ( HL : Word - Adresa ) : A - Data
; - EEPROM_Write  ( HL : Word - Adresa; A - Data )
;   WriteToEEPROM:
;    LDI H,0x00
;    LDI L,0x01
;    LDI A,xxxx
;    EEPROM_Write
;
; USART ------------------------------------------------------------- 8.
; - USART_Init
; - USART_PC
; - USART_Send ( A : Byte ) ; Caka iba kym odosle
; - USART_Read : A - Data   ; Caka kym neprijime data 
; - USART_Start
; - USART_Stop
; - USART_SkipIfReceive
; - USART_SkipIfNotReceive
; - USART_SkipIf_FarmeError
; - USART_SkipIfNot_FrameError
; - USART_SkipIf_OverRun
; - USART_SkipIfNot_OverRun
; - USART_ResetFlag_FrameError
; - USART_ResetFlag_OverRun
;   InitializationUSART:
;    USART_Init
;    LDI A, 0xXX
;    USART_Send 

; == Body nastavenia ===================================================
; 2.B - Port B 
; 2.C - Port C 
; 2.D - Port D 
; 3.D - Preddelicka 16. bit casovaca/Timera 
; 3.E - Preddelicka 8.  bit casovaca/Timera 
; 4.A - Nastavenie portu pre 595 posuvny register
; 6.A - Prednastavenie AD prevodnika
; 8.A - Nastavenie USART pre DMX512
; 8.B - Nastavenie USART pre PC 

; Nastavenie kompilatora pre ATMega8
.device ATmega8

; Oznacenie --------------------------------------------------------- 1.
; - Status register
.equ	SREG	=$3f
; - Zasobnik
.equ	SPH		=$3e
.equ	SPL		=$3d
.equ 	RAMEND	=$45F
.equ 	RAMENDH	=$04
.equ 	RAMENDL	=$5F
; - Hlavne nastavenie MCU
.equ    SFIOR   =$30
.equ    PUD     =2
; - PORTY
.equ	PORTB	=$18
.equ	DDRB	=$17
.equ	PINB	=$16
.equ	PORTC	=$15
.equ	DDRC	=$14
.equ	PINC	=$13
.equ	PORTD	=$12
.equ	DDRD	=$11
.equ	PIND	=$10
; - Citac/Timer
.equ	TIMSK	=$39
.equ	TCCR1A	=$2f
.equ	TCCR1B	=$2e
.equ	TCNT1H	=$2d
.equ	TCNT1L	=$2c
.equ	OCR1AH	=$2b
.equ	OCR1AL	=$2a
.equ	OCR1BH	=$29
.equ	OCR1BL	=$28
.equ	ICR1H	=$27
.equ	ICR1L	=$26
.equ	TCCR2	=$25
.equ	TCNT2	=$24
.equ	OCR2	=$23
; - Interupt
.equ    MCUCR   =$35
.equ	GIMSK	=$3b
; - AD prevodnik
.equ    ADMUX   =$07
.equ    ADCSR   =$06
.equ	ADCSRA	=$06
.equ    ADCH    =$05
.equ    ADCL    =$04
.equ    ADEN    =7
.equ    ADSC    =6
.equ    ADFR    =5
.equ    ADIF    =4
.equ    ADIE    =3
.equ    ADPS2   =2
.equ    ADPS1   =1
.equ    ADPS0   =0
; - EEPROM
.equ	EEARH	=$1f
.equ	EEARL	=$1e
.equ	EEDR	=$1d
.equ	EECR	=$1c
.equ    EERIE   =3
.equ	EEMWE	=2
.equ	EEWE	=1
.equ	EERE	=0
; - USART
.equ	UDR		=$0c
.equ	UCSRA	=$0b
.equ	UCSRB	=$0a
.equ	UCSRC	=$20		;  Note! UCSRC equals UBRRH
.equ	UBRRL	=$09
.equ	RXC		=7
.equ	TXC		=6
.equ	UDRE	=5
.equ	FE		=4
.equ	DOR		=3
.equ	UPE		=2
.equ	PE		=2
.equ	U2X		=1
.equ	MPCM	=0
.equ	RXCIE	=7
.equ	TXCIE	=6
.equ	UDRIE	=5
.equ	RXEN	=4
.equ	TXEN	=3
.equ	CHR9	=2		; old name kept for compatibilty
.equ	UCSZ2	=2
.equ	RXB8	=1
.equ	TXB8	=0
; - Registre
.def	A		=r16	; Akumulator
.def	B		=r17	; Pomocny akumulator
.def	T		=r18	; Temp
.def	F		=r19	; Status Flag
.def	Z		=r30	; Z Register	
.def	XL		=r26
.def	XH		=r27
.def	YL		=r28
.def	YH		=r29
.def	ZL		=r30
.def	ZH		=r31
.def	HL		=r24	; Adresa
.def	L		=r24
.def	H		=r25
; R6..13 je pridruzene bufferu


; Tabulka Preruseni
.org	$000
 RJMP	Initialization;
;.equ	INT0addr=$001	; External Interrupt0 Vector Address
.org	$001
 RJMP	NilInterupt;
;.equ	INT1addr=$002	; External Interrupt1 Vector Address
.org	$002
 RJMP	NilInterupt;
;.equ	OC2addr =$003	; Output Compare2 Interrupt Vector Address
.org	$003
 RJMP	NilInterupt;
;.equ	OVF2addr=$004	; Overflow2 Interrupt Vector Address
.org	$004
 RJMP	NilInterupt;
;.equ	ICP1addr=$005	; Input Capture1 Interrupt Vector Address
.org	$005
 RJMP	NilInterupt;
;.equ	OC1Aaddr=$006	; Output Compare1A Interrupt Vector Address
.org	$006
 RJMP	Timer_Interupt;
;.equ	OC1Baddr=$007	; Output Compare1B Interrupt Vector Address
.org	$007
 RJMP	NilInterupt;
;.equ	OVF1addr=$008	; Overflow1 Interrupt Vector Address
.org	$008
 RJMP	NilInterupt;
;.equ	OVF0addr=$009	; Overflow0 Interrupt Vector Address
.org	$009
 RJMP	NilInterupt;
;.equ	SPIaddr =$00a	; SPI Interrupt Vector Address
.org	$00a
 RJMP	NilInterupt;
;.equ	URXCaddr=$00b	; USART Receive Complete Interrupt Vector Address
.org	$00b
 RJMP	USART_Interupt;
;.equ	UDREaddr=$00c	; USART Data Register Empty Interrupt Vector Address
.org	$00c
 RJMP	NilInterupt;
;.equ	UTXCaddr=$00d	; USART Transmit Complete Interrupt Vector Address
.org	$00d
 RJMP	NilInterupt;
;.equ	ADCCaddr=$00e	; ADC Interrupt Vector Address
.org	$00e
 RJMP	NilInterupt;
;.equ	ERDYaddr=$00f	; EEPROM Interrupt Vector Address
.org	$00f
 RJMP	NilInterupt;
;.equ	ACIaddr =$010	; Analog Comparator Interrupt Vector Address
.org	$010
 RJMP	NilInterupt;
;.equ   TWIaddr =$011   ; Irq. vector address for Two-Wire Interface
.org	$011
 RJMP	NilInterupt;
;.equ	SPMaddr =$012	; SPM complete Interrupt Vector Address
.org	$012
 RJMP	NilInterupt;

; Nepriradeni skok
NilInterupt:
 Reti;

; Makro zalohovania pri preruseni
.Macro PushA
 Push	T
 Push	A
 Push	B
; Push	ZH
; Push 	ZL
 In		A		,SREG
 Push	A
 Cli
.EndMacro
.Macro PopA
 Pop	A
 Out	SREG	, A
 ;Pop	ZL
 ;Pop 	ZH
 Pop 	B
 Pop	A
 Pop	T
.EndMacro

; Inicializacia MCU ------------------------------------------------- 2. 
Initialization:
 ; Nastavenie StackPointera
 Ldi	T		, RamEndH
 Out	SPH		, T			
 Ldi	T		, RamEndL			
 Out	SPL		, T
 ; Zapnutie Pull-Up rezistorov 
 In 	T		, SFIOR
 Andi 	T		, 0b11111011
 Out 	SFIOR	, T
 ; Nastavenie portu B na vystup ------------------------------------ 2.B	
 ; - Priepustnost 0-Vstup/1-Vystup		
 Ldi 	T		, 0b00000000	
 Out 	DdrB	, T
 ; - Vystup/Pull-Up
 Ldi 	T		, 0b00000000    
 Out 	PortB	, T
 ; Nastavenie portu C na vystup ------------------------------------ 2.C	
 ; - Priepustnost 0-Vstup/1-Vystup		
 Ldi 	T		, 0b00011111	
 Out 	DdrC	, T
 ; - Vystup/Pull-Up
 Ldi 	T		, 0b00100000    
 Out 	PortC	, T
 ; Nastavenie portu D na vystup ------------------------------------ 2.D	
 ; - Priepustnost 0-Vstup/1-Vystup
 Ldi 	T		, 0b11110000	
 Out 	DdrD	, T
 ; - Vystup/Pull-Up
 Ldi 	T		, 0b00000000    
 Out 	PortD	, T
; Pri in icializacii 595 sa zmenia :
;   - 7,6,5 -> na vystup		

; Skok na koniec  
 Rjmp EndOfInitFile
 
; Citace ------------------------------------------------------------ 3. 
.Macro Timer_Init; ( HL : Word ) - HL je TOP Citaca. @8Mhz -> 1Mhz / HL
 ; Nastavenia zdroju signalu TCCR1B
 ; 0 - Preddelicka :
 ; 1 - 0, /1, /8, /64, /256, /1024, T1, nT1
 ; 2 - 
 ; -----------------------------------------
 ; 3 - 1 rezim casovaca
 ; 4 - 0
 ; -----------------------------------------
 ; 5 -
 ; 6 -
 ; 7 -
 Ldi 	T		, 0b00001010   	; predelicka na /8 ----------------- 3.D   		 
 Out 	TCCR1B	, T	
 ; Nastavenia zdroju signalu TCCR1A
 Ldi 	T		, 0b00000000       		 
 Out 	TCCR1A	, T	
 ; Nastavenie prerusenia TIMSK
 ; prerusenie od OCIE1A
 In		T		, TIMSK
 Ori 	T		, 0b00010000       		
 Out 	TIMSK	, T
 ; Reset Pocitadla
 Clr	T	
 Out	TCNT1H	, T     		; Horny byte
 Out	TCNT1L	, T     		; Dolny byte
 ; Nastavenie hodnoty pre register OCR1A
 Out 	OCR1AH	, H     		; Horny byte
 Out 	OCR1AL	, L     		; Dolny byte
.EndMacro 
; Reset Timera 
.Macro Timer_Reset
 Clr	T	
 Out	TCNT1H	, T     		; Horny byte
 Out	TCNT1L	, T     		; Dolny byte
.EndMacro 
.Macro Timer_Div1
 Ldi 	T		, 0b00001001   	   		 
 Out 	TCCR1B	, T	
.EndMacro 
.Macro Timer_Div8
 Ldi 	T		, 0b00001010   	   		 
 Out 	TCCR1B	, T	
.EndMacro 
.Macro Timer_Div64
 Ldi 	T		, 0b00001011   	   		 
 Out 	TCCR1B	, T	
.EndMacro 
.Macro Timer_Div256
 Ldi 	T		, 0b00001100   	   		 
 Out 	TCCR1B	, T	
.EndMacro 
.Macro Timer_Div1024
 Ldi 	T		, 0b00001101   	   		 
 Out 	TCCR1B	, T	
.EndMacro 
; Poznamka:
; 16Mhz : 8		 = 2Mhz Preddelicka
; 2Mhz  : 20 000 = 100Hz
;         20.000 = 4E 20
; Druhy 8bit citac
.Macro Timer8_Init; ( L : Byte ) - L je TOP Citaca. @8Mhz -> 1Mhz / L
 ; Nastavenia zdroju signalu TCCR2
 ; 0 - Preddelicka :
 ; 1 - 0, /1, /8, /32, /64, /128, /256, /1024
 ; 2 - 
 ; -----------------------------------------
 ; 3 - 1 rezim casovaca
 ; 4 - 0
 ; 5 - 0
 ; 6 - 0 rezim casovaca
 ; 7 - 0
 Ldi 	T		, 0b00001010   	; predelicka na /8 ----------------- 3.E   		 
 Out 	TCCR2	, T	
 ; Nastavenie prerusenia TIMSK
 ; prerusenie od OCIE1A
 In		T		, TIMSK
 Ori 	T		, 0b10000000       		
 Out 	TIMSK	, T
 ; Reset Pocitadla
 Clr	T	
 Out	TCNT2	, T     		; Dolny byte
 ; Nastavenie hodnoty pre register OCR1A
 Out 	OCR2	, L     		; byte
.EndMacro 
; Reset Timera 
.Macro Timer8_Reset
 Clr	T	
 Out	TCNT2	, T     	
.EndMacro 
.Macro Timer8_Div1
 Ldi 	T		, 0b00001001   	   		 
 Out 	TCCR2	, T	
.EndMacro 
.Macro Timer8_Div8
 Ldi 	T		, 0b00001010   	   		 
 Out 	TCCR2	, T	
.EndMacro 
.Macro Timer8_Div32
 Ldi 	T		, 0b00001011   	   		 
 Out 	TCCR2	, T	
.EndMacro 
.Macro Timer8_Div64
 Ldi 	T		, 0b00001100   	   		 
 Out 	TCCR2	, T	
.EndMacro 
.Macro Timer8_Div128
 Ldi 	T		, 0b00001101   	   		 
 Out 	TCCR2	, T	
.EndMacro 
.Macro Timer8_Div256
 Ldi 	T		, 0b00001110   	   		 
 Out 	TCCR2	, T	
.EndMacro 
.Macro Timer8_Div1024
 Ldi 	T		, 0b00001111   	   		 
 Out 	TCCR2	, T	
.EndMacro 

; 74xxx595 Posuvny register ----------------------------------------- 4.
;  - Data    - - Data pusuvneho registra  
;  - Clk     - - Casove impulzi posuvneho registra
;  - Strobe  - - Impulz prenosu zo posuvneho do paralneho registra
; Nastavenie vysupneho portu --------------------------------------- 4.A  
.Equ	d595	= 5			; Data   - pin portu D
.Equ	c595	= 6			; Clock  - pin portu D
.Equ	s595	= 7			; Strobe - pin portu D
.Equ	p595	= $12		; Port D 

.Macro 595_Init ;
 Push	T
 ; Nastavenie I/O na vystup
 In		T		, p595+1
 Set
 Bld	T		, d595
 Bld	T		, c595
 Bld	T		, s595
 Out	p595+1	, T 
 ; Vynulovanie vystupov
 In		T		, p595
 Clt
 Bld	T		, d595
 Bld	T		, c595
 Bld	T		, s595
 Out	p595	, T
 Pop	T
.EndMacro
; Clock impulz
.Macro 595_Clk
 set					; Generovanie inpulzu Clk
 bld	T		, c595	; 
 out	p595	, T		; 
 clt					; Ukoncenie inpulzu Clk
 bld	T		, c595	; 
 out	p595	, T		; 
.EndMacro
; Vyslanie 8. bitoveho paketu
.Macro 595_Send_Macro ; ( A : Byte ) A je parameter ktory sa odosle
 In		T		, p595
 ; Odoslanie bitov
 bst	A		, 7		; Vyber bitu na odoslanie 
 bld	T		, d595	;
 out	p595	, T		; 
 595_Clk
 bst	A		, 6		; Vyber bitu na odoslanie 
 bld	T		, d595	;
 out	p595	, T		; 
 595_Clk
 bst	A		, 5		; Vyber bitu na odoslanie 
 bld	T		, d595	;
 out	p595	, T		; 
 595_Clk
 bst	A		, 4		; Vyber bitu na odoslanie 
 bld	T		, d595	;
 out	p595	, T		; 
 595_Clk
 bst	A		, 3		; Vyber bitu na odoslanie 
 bld	T		, d595	;
 out	p595	, T		; 
 595_Clk
 bst	A		, 2		; Vyber bitu na odoslanie 
 bld	T		, d595	;
 out	p595	, T		; 
 595_Clk
 bst	A		, 1		; Vyber bitu na odoslanie 
 bld	T		, d595	;
 out	p595	, T		; 
 595_Clk
 bst	A		, 0		; Vyber bitu na odoslanie 
 bld	T		, d595	;
 out	p595	, T		; 
 595_Clk
 ; Koniec. Po poslednom poslani treba vyslat impulz Stb
.EndMacro
 ; Fyzicky kod
w595_Send:
 595_Send_Macro
 Ret
 ; Volanie fyzickeho kodu zec makro
.Macro 595_Send
 RCall w595_Send
.EndMacro 
; Vykreslenie data na vystup
.Macro 595_Done_Macro
 In 	T		, p595
 set					; Generovanie inpulzu Stb
 bld	T		, s595	; 
 out	p595	, T		; 
 clt					; Ukoncenie inpulzu Stb
 bld	T		, s595	; 
 out	p595	, T		; 
.EndMacro
 ; Fyzicky kod
w595_Done:
 595_Done_Macro
 Ret
 ; Volanie fyzickeho kodu zec makro
.Macro 595_Done
 RCall w595_Done
.EndMacro 
 
; Externeho prerusenie ---------------------------------------------- 5.
; - Prerusenie od INT0 
.Macro Int0_Enable
 In		T		, MCUCR
 ;Andi	T		, 0b11111100
 Ori	T		, 0b00000011
 Out	MCUCR	, T
 In		T		, GIMSK
 Ori	T		, 0b01000000
 Out	GIMSK	, T 
.EndMacro
.Macro Int0_Disable
 In		T		, GIMSK
 Andi	T		, 0b10111111
 Out	GIMSK	, T 
.EndMacro
.Macro Int0_OnH
 In		T		, MCUCR
 ;Andi	T		, 0b11111100
 Ori	T		, 0b00000011
 Out	MCUCR	, T
.EndMacro
.Macro Int0_OnL
 In		T		, MCUCR
 Andi	T		, 0b11111100
 Ori	T		, 0b00000010
 Out	MCUCR	, T
.EndMacro
.Macro Int0_OnChange
 In		T		, MCUCR
 Andi	T		, 0b11111100
 Ori	T		, 0b00000001
 Out	MCUCR	, T
.EndMacro
; - Preruseenie od INT1
.Macro Int1_Enable
 In		T		, MCUCR
 ;Andi	T		, 0b11110011
 Ori	T		, 0b00001111
 Out	MCUCR	, T
 In		T		, GIMSK
 Ori	T		, 0b10000000
 Out	GIMSK	, T 
.EndMacro
.Macro Int1_Disable
 In		T		, GIMSK
 Andi	T		, 0b01111111
 Out	GIMSK	, T 
.EndMacro
.Macro Int1_OnH
 In		T		, MCUCR
 ;Andi	T		, 0b11110011
 Ori	T		, 0b00001100
 Out	MCUCR	, T
.EndMacro
.Macro Int1_OnL
 In		T		, MCUCR
 Andi	T		, 0b11110011
 Ori	T		, 0b00001000
 Out	MCUCR	, T
.EndMacro
.Macro Int1_OnChange
 In		T		, MCUCR
 Andi	T		, 0b11110011
 Ori	T		, 0b00000100
 Out	MCUCR	, T
.EndMacro

; AD Prevodnik ------------------------------------------------------ 6.

; Nastavenie regista ADMUX
; 0 - Mux 0    Tabulka:
; 1 - Mux 1    ADC0,1,2,3,4,5,6,7,6xRez,1.23V,GND
; 2 - Mux 2
; 3 - Mux 3
; 4 - Rezerva
; 5 - ADLAR    0: L=0-7 H=8-9; 1: L=0-1 H=2-9
; 6 - REFS 0   Tabulka:
; 7 - REFS 1   Gnd, AVcc, Reserva, 2,56V

; Inicializacia 8bit/5V/Pin0. -------------------------------------- 6.A
.MACRO AD_Init	
 Ldi	T		, 0b01100000 
 Out	ADMUX	, T
.ENDMACRO 
; Vyber Pinu
.MACRO AD_Mux	; ( A : Byte - Parameter vstupny Pin )
 In		T		, ADMUX
 Andi	T		, 0b11110000
 Or		T		, A 
 Out	ADMUX	, T
.ENDMACRO 
; Nacitanie parametra
.MACRO AD_Read 			; A=Result  
 IN		A		, ADCL
 IN		A		, ADCH
.ENDMACRO
; Udaj Do akumulatora
.MACRO AD_Read10		; HL=Result  
 IN		L		, ADCL
 IN		H		, ADCH
.ENDMACRO
; Utility a nastavenia 
.MACRO AD_10bit	
 In		T		, ADMUX
 Andi	T		, 0b11011111 
 Out	ADMUX	, T
.ENDMACRO 
.MACRO AD_8bit	
 In		T		, ADMUX
 Ori	T		, 0b00100000 
 Out	ADMUX	, T
.ENDMACRO 
.MACRO AD_Ref5V	
 In		T		, ADMUX
 Andi	T		, 0b00111111 
 Ori	T		, 0b01000000 
 Out	ADMUX	, T
.ENDMACRO 
.MACRO AD_Ref2V56	
 In		T		, ADMUX
 Andi	T		, 0b00111111 
 Ori	T		, 0b11000000 
 Out	ADMUX	, T
.ENDMACRO 
; Ovladanie pomocov paramer ADCSRA
.MACRO	AD_Reset
 Ldi	T		, 0b10000000
 Out	ADCSRA	, T
.ENDMACRO 
; Start konverzie 
.MACRO	AD_Start
 Sbi	ADCSRA	, ADSC
.ENDMACRO
; Pockaj kym neskonci
.MACRO	AD_Wait  
 mADWaitLoop:
 Sbis 	ADCSRA	, ADIF
 Rjmp	mADWaitLoop
.ENDMACRO
 ; Procedura ADRead ( A - Vsstupny pin ) : A - Vystupna hodnota AD
 wAD_Read:
  AD_Mux			; Nastavenie vstupneho pinu
  AD_Reset			; Vynulovanie Redistra a spustenie ADPrevodniku
  AD_Start			; Spustenie konverzie
  AD_Wait			; Pocka kym to spravy 
  AD_Read			; Precitanie nameranej hodnoty
  RET
 ; Ako Makro 
 .MACRO	AD_Convert   
  RCALL	wAD_Read
 .ENDMACRO 
 ; Procedura ADRead10 ( A - Vstupny pin ) : HL - Vystupna hodnota AD
 wAD_Read10:
  AD_Mux			; Nastavenie vstupneho pinu
  AD_Reset			; Vynulovanie Redistra a spustenie ADPrevodniku
  AD_Start			; Spustenie konverzie
  AD_Wait			; Pocka kym to spravy 
  AD_Read10			; Precitanie nameranej hodnoty
  RET
 ; Ako Makro 
 .MACRO	AD_Convert10   
  RCALL	wAD_Read10
 .ENDMACRO 

; EEPROM ------------------------------------------------------------ 7.
; Cakanie, kym je pamat pripravena
.Macro EEPROM_Wait
eeWait:
 Sbic	EECR	, EEWE			; Ak je eeprom zanepraznena
 Rjmp	eeWait					; Cakaj!
.EndMacro
; Citanie
.Macro EEPROM_Read ; ( HL : Word - Adreasa; A : Byte - Data )
 EEPROM_Wait
 Out	EEARH	, H				; Vyslanie do EEPROM
 Out	EEARL	, L				; Vyslanie do EEPROM
 Sbi	EECR	, EERE			; Vlajka citania
 In		A		, EEDR			; Nacitanie spodneho bytu
.EndMacro
; Zapis
.Macro EEPROM_Write ; ( HL : Word - Adreasa) : A  - Data 
 EEPROM_Wait
 In		T		, SREG
 Cli
 Out	EEARH	, H				; Vyslanie do EEPROM
 Out	EEARL	, L				; Vyslanie do EEPROM
 Out	EEDR	, A				; Vyslanie do EEPROM
 Sbi	EECR	, EEMWE			; Vlajka zapisu
 Sbi	EECR	, EEWE			; Vlajka zapisu
 Out	SREG	, T
.EndMacro

; USART ------------------------------------------------------------- 8.
.Macro USART_Init ; @8Mhz / 250Kbps / 8bit+2Stop / no parity - DMX512
 Ldi	T		, 0x00
 Out	UCSRC	, T
 Ldi	T		, 3 		; Nadtavenie rychlosti ----------------- 8.A
 Out	UBRRL	, T			; - Pre @16Mhz je hodnota 3
 ; === UCSRB ===
 ; 7. RXCIE		- Povolenie prerusenia od dokoncenia prijimania
 ; 6. TXCIE		- Povolenie prerusenia od dokoncenia Vysielania
 ; 5. UDRIE
 ; ------------------------------------------------------------
 ; 4. RXEN 		- Povolenie Prijimania
 ; 3. TXEN     	- Povolenie Vysielania
 ; ------------------------------------------------------------
 ; 2. UCSZ2		- Povolenie 9 bitu
 ; 1. RXB8		- 9bit prijimaca
 ; 0. TXB8		- 9bit vysielaca
 Ldi	T		, 0b10010000; Povolenie Vysielania/prijimania
 Out	UCSRB	, T
 ; === UCSRC ===
 ; 7. URSEL = 1
 ; 6. UMSEL		- 1-Synchronizovany/0-Asynchronizovany
 ; -------------------------------------------------------------
 ; 5. UPM1		- Parita:
 ; 4. UPM0		- Disable/Reserved/Even(parny)/Odd(Neparny)
 ; -------------------------------------------------------------
 ; 3. USBS		- Stop bit. 0-1x / 1-2x
 ; -------------------------------------------------------------
 ; 2. UCSZ1		- Pocet bitov:
 ; 1. UCSZ0		- 5,6,7,8 Bit(UCSZ2=1: Reserved/Reserved/Reserved/9Bit)
 ; -------------------------------------------------------------
 ; 0. UCPOL		- Polarita : 0-nabezna / 1-klesajuca
 Ldi	T		, 0b10000110; 
 Out	UCSRC	, T 
.EndMacro 
.Macro USART_PC ; @8Mhz / 9600 / 8bit+1Stop / no parity 
 Ldi	T		, 0x00
 Out	UCSRC	, T
 Ldi	T		, 51 		; Nadtavenie rychlosti ----------------- 8.B
 Out	UBRRL	, T			; - Pre @16Mhz je hodnota 103
 Ldi	T		, 0b00011000; Povolenie Vysielania/prijimania
 Out	UCSRB	, T
 Ldi	T		, 0b10000110; 
 Out	UCSRC	, T 
.EndMacro 
; Odosielanie a prijem
.Macro	USART_Send ; ( A - Data )
 wUSART_WaitLoopS:
 Sbis	UCSRA	, UDRE
 Rjmp	wUSART_WaitLoopS
 Out	UDR		, A
.EndMacro
.Macro	USART_Read ; : A - Data 
 wUSART_WaitLoopR:
 Sbis	UCSRA	, RXC
 Rjmp	wUSART_WaitLoopR
 In		A		, UDR
.EndMacro
 ; === UCSRA === STATOVY Register ====
 ; 7. RXC		- Prijatie ukoncene
 ; 6. TXC		- Vysielanie dokoncene
 ; 5. UDRE		- Vysielaci register je prazny
 ; -------------------------------------------------------------
 ; 4. FE		- Frame Error
 ; 3. DOR		- Data OverRun
 ; 2. PE		- Parity Error
 ; -------------------------------------------------------------
 ; 1. U2X		- Dvojnasobna rychlost
 ; 0. MPCM		- Multyporocesor mode
 ; Utility
.Macro USART_Start
 Sbi	UBRRL	, RXEN
.EndMacro
.Macro USART_Stop
 Cbi	UBRRL	, RXEN
.EndMacro
.Macro USART_SkipIfReceive
 Sbis	UCSRA	, RXC
.EndMacro
.Macro USART_SkipIfNotReceive
 Sbic	UCSRA	, RXC
.EndMacro
.Macro USART_SkipIf_FrameError
 Sbis	UCSRA	, FE
.EndMacro
.Macro USART_SkipIfNot_FrameError
 Sbic	UCSRA	, FE
.EndMacro
.Macro USART_SkipIf_DataOverRun
 Sbis	UCSRA	, DOR
.EndMacro
.Macro USART_SkipIfNot_DataOverRun
 Sbic	UCSRA	, DOR
.EndMacro
.Macro USART_ResetFlag_FrameError
 Cbi	UCSRA	, FE
.EndMacro
.Macro USART_ResetFlag_DataOverRun
 Cbi	UCSRA	, DOR
.EndMacro

; Koniec inicializacie. 
EndOfInitFile: 




